/*
 * Copyright (C) 2011-2012 jeanfi@gmail.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#include <libintl.h>
#define _(String) gettext(String)

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <json/json.h>

#include "cache.h"
#include "fcache.h"
#include "http.h"
#include "list.h"
#include "log.h"
#include "lp_ws.h"
#include "lp_json.h"
#include "ppastats.h"

static const char *
QUERY_GET_PUBLISHED_BINARIES = "?ws.op=getPublishedBinaries&ws.size=150";
static const char *QUERY_GET_DOWNLOAD_COUNT = "?ws.op=getDownloadCount";
static const char *
QUERY_GET_DAILY_DOWNLOAD_TOTALS = "?ws.op=getDailyDownloadTotals";

static json_object *get_json_object(const char *url)
{
	json_object *obj = NULL;
	char *body;

	body = get_url_content(url, 0);

	if (body) {
		obj = json_tokener_parse(body);

		free(body);

		return obj;
	}

	return NULL;
}

struct bpph **get_bpph_list(const char *archive_url, const char *pkg_status)
{
	char *url, *bpph_key;
	struct bpph **result = NULL;
	struct json_object *o, *bpph_json, *o_next;

	url = malloc(strlen(archive_url)+
		     strlen(QUERY_GET_PUBLISHED_BINARIES)+
		     (pkg_status ? strlen("&status=")+strlen(pkg_status) : 0)+
		     1);

	strcpy(url, archive_url);
	strcat(url, QUERY_GET_PUBLISHED_BINARIES);

	if (pkg_status) {
		strcat(url, "&status=");
		strcat(url, pkg_status);
	}

	while (url) {
		o = get_json_object(url);
		free(url);
		url = NULL;

		if (!o)
			break;

		result = (struct bpph **)list_append_list
			((void **)result,
			 (void **)json_object_to_bpph_list(o));

		o_next = json_object_object_get(o, "next_collection_link");

		if (o_next)
			url = strdup(json_object_get_string(o_next));

		json_object_put(o);
	}

	bpph_json = bpph_list_to_json(result);

	bpph_key = malloc(strlen(archive_url + 7) + strlen("/bpph") + 1);
	sprintf(bpph_key, "%s/bpph", archive_url + 7);
	fcache_put(bpph_key, json_object_to_json_string(bpph_json));

	json_object_put(bpph_json);
	free(bpph_key);

	return result;
}

int get_download_count(const char *archive_url)
{
	int n = strlen(archive_url) + strlen(QUERY_GET_DOWNLOAD_COUNT) + 1;
	char *url = malloc(n);
	int result;
	json_object *obj;

	strcpy(url, archive_url);
	strcat(url, QUERY_GET_DOWNLOAD_COUNT);

	obj = get_json_object(url);
	free(url);

	if (!obj)
		return -1;

	result = json_object_get_int(obj);

	json_object_put(obj);

	return result;
}

const struct distro_arch_series *get_distro_arch_series(const char *url)
{
	json_object *obj;
	const struct distro_arch_series *distro;
	char *content;

	distro = cache_get(url);
	if (distro)
		return (struct distro_arch_series *)distro;

	content = get_url_content(url, 1);

	if (!content)
		return NULL;

	obj = json_tokener_parse(content);

	free(content);

	if (!obj)
		return NULL;

	distro = json_object_to_distro_arch_series(obj);

	json_object_put(obj);

	cache_put(url, distro, (void (*)(void *))&distro_arch_series_free);

	return distro;
}

const struct distro_series *get_distro_series(const char *url)
{
	json_object *obj;
	const struct distro_series *distro;
	char *content;

	distro = cache_get(url);
	if (distro)
		return (struct distro_series *)distro;

	content = get_url_content(url, 1);

	if (!content)
		return NULL;

	obj = json_tokener_parse(content);

	free(content);

	if (!obj)
		return NULL;

	distro = json_object_to_distro_series(obj);

	json_object_put(obj);

	cache_put(url, distro, (void (*)(void *))&distro_series_free);

	return distro;
}

struct daily_download_total **get_daily_download_totals(const char *binary_url)
{
	char *url;
	json_object *obj;
	struct daily_download_total **result = NULL;

	url = malloc(strlen(binary_url)+
		     strlen(QUERY_GET_DAILY_DOWNLOAD_TOTALS)+1);

	strcpy(url, binary_url);
	strcat(url, QUERY_GET_DAILY_DOWNLOAD_TOTALS);

	obj = get_json_object(url);

	if (obj) {
		result = json_object_to_daily_download_totals(obj);
		json_object_put(obj);
	}

	free(url);

	return result;
}

